<?php
/**
 * @file
 * Contains Drupal\partial_access\EventSubscriber\RestrictedContentSubscriber.
 */
namespace Drupal\partial_access\EventSubscriber;

use Drupal\Core\Database\Database;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\node\NodeInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\user\Entity\User;

/**
 * Class RestrictedContentSubscriber.
 *
 * @package Drupal\partial_access\EventSubscriber
 */
class RestrictedContentSubscriber implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;


  /**
   * Constructs the subscriber.
   */
  public function __construct(AccountProxyInterface $current_user, ConfigFactoryInterface $config_factory) {
      $this->currentUser = $current_user;
      $this->configFactory = $config_factory;
  }

public function checkAndRemoveExpiredRole($uid) {
  if (!$uid) {
    return;
  }

  // 1. Check for an active subscription for this user.
  $connection = \Drupal::database();
  $record = $connection->select('user_subscriptions', 'us')
    ->fields('us', ['plan_id'])
    ->condition('uid', $uid)
    ->condition('end', \Drupal::time()->getRequestTime(), '>')
    ->range(0, 1)
    ->execute()
    ->fetchAssoc();

  // 2. If no active subscription, try to remove any privileged role.
  if (!$record && isset($record['plan_id'])) {
    $user = User::load($uid);

    // Optionally, you can loop through all plans and remove any possible privileged role.
    // But if you want to remove only the last assigned, you may want to keep track of it.
    // Here, let's remove all possible privileged roles from all plans:
    $plan = \Drupal::entityTypeManager()->getStorage('subscription_plan')->load($record['plan_id']);
    if ($plan && !empty($plan->stripe_subscription_role) && $user->hasRole($plan->stripe_subscription_role)) {
      $user->removeRole($plan->stripe_subscription_role);
    }
    $user->save();
    return;
  }
}  

  /**
   * Responds to the view event for nodes.
   *
   * @return void
   */
  public function checkContentAccess(ViewEvent $event) {
    $route_match = \Drupal::routeMatch();
    $node = $route_match->getParameter('node');
    if ($node instanceof NodeInterface && $node->bundle() === 'blog') {
      $roles = $this->currentUser->getRoles();
      // Load config settings
      $config = $this->configFactory->get('restricted_content.settings');
      $cta_text = $config->get('cta_text') ?? 'Upgrade to Full Access';
      $cta_url = $config->get('cta_url') ?? '/subscribe';

      $current_user_id = $this->currentUser->id();
      $this->checkAndRemoveExpiredRole($current_user_id);
      $author_id = $node->getOwnerId();

      // Load the content author.
      $author = User::load($author_id);
      $restricted_roles = array_keys(array_filter($config->get('restricted_roles') ?? []));
      $access_roles = array_keys(array_filter($config->get('access_roles') ?? []));
      // Check if the current user has any of the restricted roles.
      if (empty(array_intersect($roles, $restricted_roles))) {
          // Allow full access to editors if they are the creator.
          if ($current_user_id === $author_id) {
              return;
          }

          // Allow full content access to paid users.
          if (array_intersect($roles, $access_roles)) {
              return;
          }

          // Others see partial content with an upgrade button.
          if ($node->hasField('body')) {
              $body = $node->get('body');
              $body_value = $body->value;
              // Strip HTML tags to avoid breaking paragraphs.
              $plain_text = trim(strip_tags($body_value));

              // Set percentage to show (you can also get this from config if needed)
              $percentage_to_show = $config->get('body_percentage');
              $total_length = mb_strlen($plain_text);
              $chars_to_show = max(1, (int) ceil($total_length * $percentage_to_show / 100));

              $partial_content = mb_substr($plain_text, 0, $chars_to_show);

              $upgrade_button = '<div class="text-center"><a href="' . $cta_url . '" class="btn btn-primary" style="margin-left:10px;">' . $cta_text . '</a></div>';

              // Everything in a single <p> for inline display.
              $final_content = '<p>' . $partial_content . '... ' . $upgrade_button . '</p>';

              $node->set('body', [
                  'value' => $final_content,
                  'format' => 'full_html',
              ]);
          }

          // Prevent the full content from being displayed.
          if ($node->hasField('field_channel')) {
            $node->set('field_channel', []);
          }
          if ($node->hasField('field_tags')) {
              $node->set('field_tags', []);
          }

      }
    }
  }


  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::VIEW => ['checkContentAccess', 1],
    ];
  }
}